use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Alders
Alders'
BOM
Code
Dave
Eilam
Eilam's
Fowler
Greg
Inc
Jekyll
MAXMIND
MAXMIND's
Mark
MaxMind
MaxMind's
Oschwald
Oschwald's
PayPal
Plugin
Rolsky
Rolsky's
TidyAll
YAMLFrontMatter
al
autarch
goschwald
lib
mfowler
tidyall
tidyallrc
validator
